#ifndef MODEL_H
#define MODEL_H

#include <stdio.h>
#include <stdlib.h>
#include <QtCore/QObject>
#include <QtCore/QMutex>
#include <QtCore/QSemaphore>
#include <QtCore/QWaitCondition>
#include <QtGui/QTreeWidgetItem>
#include "voicedata.h"

#define MAX_PARAMS 10
#define MAX_SPLIT 9
#define MAX_SCENES 2
#define MAX_SOURCES 4
#define MAX_HARMONIC_GROUPS 8
#define MAX_OSC 5
#define MAX_SCALES 80
#define MAX_POLY 128
#define STEP 64
#define FILTER_LEN 48
#define MAX_LEVEL 2147483647
#define MAX_FRAMES 16384

typedef struct controlObject {
  QTreeWidgetItem *treeWidgetItem;
  QString name;
  int objectType; 
  int objectIndex;
  int modDepth;
  bool full, inverse;
} controlObject;

typedef struct midiController {
  QTreeWidgetItem *treeWidgetItem;
  QList<controlObject> connectedObjects;
  unsigned char eventChannel;
  unsigned int eventParam;
  signed int value;
  int type;
} midiController;

class Model : public QObject
{
  private:
    QList<midiController> midiControllerList;
    QList<controlObject> controlObjectList; 
    VoiceData *voiceData[MAX_POLY];
    int currentVoiceDataIndex;
    int editStart, editEnd;
    int numHarmonics;
    int harmonicSplit[MAX_SPLIT];
    double *harmonics[MAX_PARAMS];
    double *scaledHarmonics[MAX_PARAMS];
    double scaleLin[MAX_PARAMS];
    double scaleExp[MAX_PARAMS];
    double minScale[MAX_PARAMS];
    double maxScale[MAX_PARAMS];
    double scale[MAX_SCALES];
    double filter[2][FILTER_LEN];
    double filterCutoff[2], filterSpacing[2], filterKeyTracking[2], filterDepth[2];
    double filterMixMorph[MAX_SOURCES], filterMixMode[MAX_SOURCES], filterMixEnvAmount[MAX_SOURCES];
    double filterMixDelay[MAX_SOURCES], filterMixAttack[MAX_SOURCES], filterMixDecay[MAX_SOURCES];
    bool filterMixLfo[MAX_SOURCES], filterMixRetrigger[MAX_SOURCES], filterMixHold[MAX_SOURCES];
    modulationEnvelope filterModEnvelope[2];
    modulationLfo filterModLfo[2], filterModSpacingLfo[2];
    double max;
    bool verbose;
    double masterVolume, pitchBend, lfo, dlfo, lfoFreq, lfoDepth;
    double delta[6];
    double evenOdd[4];
    double noiseBandwidth;
    double morphing;
    double linearLevel, transitionWidth, compression, antialiasing;
    int lfoState;
    int editGroup;
    int rate, bufSize;
    int poly;
    voiceType *voices;
    envelopeType *envelopes[MAX_POLY];
    bool synthesisFlag;
    
  public:
    QMutex voiceMutex, synthMutex;
    QWaitCondition voiceWait, synthWait, jackWait;  
    int voiceReadyCounter;
    QSemaphore *synthSemaphore;
    double *buf[2];
    
  public:
    Model(int p_poly, QObject* parent=0);
    ~Model();

    VoiceData* getVoiceData(int index);
    VoiceData* currentVoiceData();
    void setCurrentVoiceDataIndex(int index);
    void setEditRange(int start, int end);
    void getEditRange(int& start, int& end);
    double getMax(); 
    void setMax(double p_max); 
    int getPoly(); 
    void setPoly(int p_poly); 
    int getRate(); 
    void setRate(int p_rate); 
    int getBufSize(); 
    void setBufSize(int p_bufSize); 
    double getPitchBend(); 
    void setPitchBend(double p_pitchBend); 
    double getLFO(); 
    void lfoStep(); 
    void setMasterVolume(double value);
    double getMasterVolume();
    double getMinScale(int p_index); 
    double getMaxScale(int p_index); 
    void setGroupScale(int p_group, int p_index, int value); 
    bool doSynthesis(); 
    voiceType* getVoices(); 
    envelopeType* getEnvelopes(int voiceIndex); 
    int getNumHarmonics(); 
    void setNumHarmonics(int p_numHarmonics); 
    int getEditGroup(); 
    int calcEditGroup(int p_index); 
    void setEditGroup(int p_group); 
    int getHarmonicSplit(int p_group); 
    void setHarmonicSplit(int p_group, int p_HarmonicSplit); 
    double *getHarmonics(int p_index); 
    double getHarmonic(int p_index, int p_num); 
    double getScaledHarmonic(int p_index, int p_num); 
    void setHarmonic(int p_index, int p_num, double p_value); 
    double *getFilter(int p_index);
    double getFilter(int p_index, int p_num);
    void setFilter(int p_index, int p_num, double p_value);
    double getFilterCutoff(int p_index);
    double getFilterSpacing(int p_index);
    double getFilterKeyTracking(int p_index); 
    double getFilterDepth(int p_index);
    void setFilterCutoff(int p_index, double p_value);
    void setFilterSpacing(int p_index, double p_value);
    void setFilterKeyTracking(int p_index, double p_value); 
    void setFilterDepth(int p_index, double p_value);
    void setLfoFreq(double value); 
    double getLfoFreq(); 
    void setLfoDepth(double value); 
    double getLfoDepth(); 
    void resetLFO(); 
    double getMorphing(); 
    void setMorphing(double value); 
    void setCompressor(int index, double value);
    void getCompressorCoeff(double& x0, double& x1, double& a, double& b, double& c, double& d, double& aa);
    double getDelta(int index); 
    void setDelta(int index, double value);
    void setEvenOdd(int index, double value);
    double getEvenOdd(int index);
    void setNoiseBandwidth(double value);
    double getNoiseBandwidth();
    void setFilterMixMorph(int spectrumIndex, double value);
    void setFilterMixMode(int spectrumIndex, double value);
    void setFilterMixEnvAmount(int spectrumIndex, double value);
    void setFilterMixDelay(int spectrumIndex, double value);
    void setFilterMixAttack(int spectrumIndex, double value);
    void setFilterMixDecay(int spectrumIndex, double value);
    void setFilterMixLfo(int spectrumIndex, bool on);
    void setFilterMixRetrigger(int spectrumIndex, bool on);
    void setFilterMixHold(int spectrumIndex, bool on);
    double getFilterMixMorph(int spectrumIndex);
    double getFilterMixMode(int spectrumIndex);
    double getFilterMixEnvAmount(int spectrumIndex);
    double getFilterMixDelay(int spectrumIndex);
    double getFilterMixAttack(int spectrumIndex);
    double getFilterMixDecay(int spectrumIndex);
    bool getFilterMixLfo(int spectrumIndex);
    bool getFilterMixRetrigger(int spectrumIndex);
    bool getFilterMixHold(int spectrumIndex);
    void setFilterModEnvelope(int filterIndex, modulationEnvelope p_Envelope);
    void setFilterModLfo(int filterIndex, modulationLfo p_Lfo);
    void setFilterModSpacingLfo(int filterIndex, modulationLfo p_Lfo);
    void setVerbose(bool on);
    bool getVerbose();
    modulationEnvelope getFilterModEnvelope(int filterIndex);
    modulationLfo getFilterModLfo(int filterIndex);
    modulationLfo getFilterModSpacingLfo(int filterIndex);
    void addControlObject(QString name, int objectType, int objectIndex, QTreeWidgetItem *item);
    int hasMidiController(int type, unsigned char eventChannel,
                           unsigned int eventParam);
    void addMidiController(int type, unsigned char eventChannel,
                           unsigned int eventParam, QTreeWidgetItem *item); 
    bool connectController(QTreeWidgetItem *midiControllerItem, 
                           QTreeWidgetItem *controlObjectItem);
    bool connectController(QTreeWidgetItem *midiControllerItem, 
                           int objectType, int objectIndex, int modDepth, bool full, bool inverse);
    bool disconnectController(QTreeWidgetItem *midiControllerItem, QTreeWidgetItem *controlObjectItem); 
    midiController getMidiController(int index);
    controlObject getControlObject(int index);
    int midiControllerCount();
    int connectedObjectCount(int midiControllerIndex);
    controlObject getConnectedObject(int midiIndex, int controllerIndex);
    void setControllerItem(QTreeWidgetItem *midiControllerItem,   
                           QTreeWidgetItem *controlObjectItem,    
                           QTreeWidgetItem *connectedObjectItem);
    void setControllerItem(QTreeWidgetItem *midiControllerItem,   
                           int objectType, int objectIndex,
                           QTreeWidgetItem *connectedObjectItem);
    void setControllerModulation(QTreeWidgetItem *midiControllerItem, QTreeWidgetItem *controlObjectItem, int modDepth, bool full, bool inverse);                       
    controlObject getControlObject(QTreeWidgetItem *midiControllerItem, QTreeWidgetItem *controlObjectItem, bool& success);                       
    void clearMidiControllerList();
    void clearControlObjectList();                       
    bool isMidiControllerItem(QTreeWidgetItem *midiControllerItem);
    QString getControlObjectName(int objectType, int objectIndex);
};
  
#endif
